package ch.ethz.icr.prefattach;

import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.engine.SimpleModel;
import uchicago.src.sim.gui.RectNetworkItem;
import uchicago.src.sim.network.DefaultDrawableEdge;
import uchicago.src.sim.network.DefaultDrawableNode;
import uchicago.src.sim.network.DefaultEdge;
import uchicago.src.sim.network.DefaultNode;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Preferential Attachment model
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @author Laszlo Gulyas
 * @version 2.0
 */
public class Model extends SimpleModel {
    int initNodes, m;
    int numEdges, numNodes;

    double pRemoving;
    int removingStarts;

    public void setup() {
        super.setup();
        initNodes = 3;
        m = 2;
        numEdges = 0;
        numNodes = 0;

        pRemoving = 0;
        removingStarts = 100000;
    }

    /**
     * This method creates an edge between the two given nodes.
     * Note that, as we are dealing with unidirectional graphs,
     * we only create an edge in the n1->n2 direction. This implies,
     * that we'll always need to make sure to check both the n1->n2
     * and n2->n1 cases when looking for an edge in between the two
     * nodes.
     *
     * @param n1 the starting node
     * @param n2 the ending node
     */
    protected void createEdgeBetween(DefaultNode n1, DefaultNode n2) {

        // We create the new edge
        DefaultDrawableEdge edge = new DefaultDrawableEdge(n1, n2);

        // We introduce add it to the two nodes as an in/out edge,
        // respectively
        n1.addOutEdge(edge);
        n2.addInEdge(edge);

        // We keep track of the number of edges for efficiency reasons.
        // (We'll use this to calculate the sum of degrees in the network.)
        numEdges++;
    }

    /**
     * This method creates a new node with the given ID
     *
     * @param ID the ID of the node
     * @return a new node
     */
    protected DefaultNode createNewNode(int ID) {
        // We keep track of the number of nodes
        numNodes++;

        // We create a node (which is, in fact, prepared to be displayed).
        DefaultDrawableNode n = new DefaultDrawableNode(Integer.toString(ID),
                new RectNetworkItem(10, 10));
        // We return the node created
        return n;
    }

    /**
     * Remove a node from the list of list of agents
     *
     * @param node the node to remove
     */
    protected void removeNode(DefaultNode node) {
        // Remove the edges leading out of this node
        ArrayList outEdges = node.getOutEdges();
        Iterator it = outEdges.iterator();
        while (it.hasNext()) {
            DefaultEdge edge = (DefaultEdge) it.next();
            DefaultNode other = (DefaultNode) edge.getTo();
            other.removeInEdge(edge);
        }
        node.clearOutEdges();

        // Remove all the edges that direct to this node
        ArrayList inEdges = node.getInEdges(); //(ArrayList) node.getInEdges().clone();
        it = inEdges.iterator();
        while (it.hasNext()) {
            DefaultEdge edge = (DefaultEdge) it.next();
            DefaultNode other = (DefaultNode) edge.getFrom();
            other.removeOutEdge(edge);
        }
        node.clearInEdges();

        // Remove the node itself
        agentList.remove(node);
    }

    public void buildModel() {
        super.buildModel();

        // We create the initial nodes
        for (int i = 0; i < initNodes; i++) {
            agentList.add(createNewNode(i));
        }

        // We connect all nodes to all other nodes
        Iterator it = agentList.iterator();
        while (it.hasNext()) {
            DefaultNode n = (DefaultNode) it.next();
            Iterator it2 = agentList.iterator();
            while (it2.hasNext()) {
                DefaultNode other = (DefaultNode) it2.next();
                // We know that !n.hasEdgeTo(other), since
                // the (n, other) ordered pair has not been
                // processed yet.
                if ((other != n) && (!other.hasEdgeTo(n))) {
                    createEdgeBetween(n, other);
                }
            }
        }
    }

    /**
     * This method selects an 'old' node with the appropriate probability.
     * (See Equation (77) in Albert & Barabasi, 2001.)
     *
     * @return the selected node
     */
    public DefaultNode selectNode() {

        // The node to be selected
        DefaultNode n = null;

        // The sum of degrees in the graph. We assign the value based on the
        // number of edges, instead of summing it up each time.
        int sumDegrees = numEdges * 2;

        // We draw a random number between 0.0 and 1.0
        double rand = this.getNextDoubleFromTo(0.0, 1.0);

        // We pick a node based on this random number by dynamically creating
        // a distribution based on the number of edges (and equation (77)).

        // Cumulated probability
        double cumProb = 0.0;
        boolean done = false;
        Iterator i = agentList.iterator();
        while (!done && i.hasNext()) {
            n = (DefaultNode) i.next();
            // The probability of selecting the given node.
            double prob = (double) (n.getInDegree() + n.getOutDegree()) /
                    (double) (sumDegrees);
            // The cumulated probability so far
            cumProb = cumProb + prob;
            // Does the random number falls into the interval (between the latest value
            // of cumProb and its new one)? If so, we're done!
            if (rand < cumProb)
                done = true;
        }

        // We return the selected node
        return n;
    }

    public void step() {
        // Removing nodes
        if (getTickCount() > removingStarts) {
            int numRemoved = (int) ((double) agentList.size() * pRemoving);
            System.out.println(numRemoved);

            for (int i = 0; i < numRemoved; i++) {
                int index = getNextIntFromTo(0, agentList.size() - 1);
                DefaultNode node = (DefaultNode) agentList.get(index);
                removeNode(node);
            }
        }

        // We create a new node
        DefaultNode newNode = createNewNode(numNodes + 1);

        // We create the links for it
        for (int i = 0; i < m; i++) {

            // We select the appropriate 'target' node
            DefaultNode oldNode;
            do {
                oldNode = selectNode();
            } while (newNode.hasEdgeTo(oldNode));
            // We make sure this is a new link we are trying to introduce...
            // Note that we know that !oldNode.hasEdgeTo(newNode) holds,
            // since it could not have it in previous steps and
            // it was introduced in this step, it must have been a
            // newNode -> oldNode link.


            // We create the link
            createEdgeBetween(newNode, oldNode);
        }

        // We add the new node to the list
        agentList.add(newNode);
    }


    /**
     * We print out the degree distribution
     */
    public void postStep() {

        System.out.print(getTickCount() + ": ");
        Iterator i = agentList.iterator();
        while (i.hasNext()) {
            DefaultNode n = (DefaultNode) i.next();
            System.out.print(n.getInDegree() + n.getOutDegree() + " ");
        }
        System.out.println();

    }


    public static void main(String[] args) {
        SimInit init = new SimInit();
        Model m = new Model();
        init.loadModel(m, null, false);
    }
}