package ch.ethz.icr.prefattach;

import uchicago.src.sim.analysis.Plot;
import uchicago.src.sim.analysis.NetSequenceGraph;
import uchicago.src.sim.engine.Controller;
import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.network.DefaultNode;
import uchicago.src.reflector.ListPropertyDescriptor;

import java.util.Iterator;
import java.util.Hashtable;

/**
 * Preferential Attachment model
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @author Laszlo Gulyas
 * @version 2.0
 */
public class ModelGUI extends Model {
    private static final int LAYOUT_RANDOM = 0;
    private static final int LAYOUT_CIRCULAR = 1;
    private static final int LAYOUT_FRUCHT = 2;
    private static final int LAYOUT_KAMADA = 3;

    boolean showPowerPlot;
    private Plot powerPlot;
    boolean showGraph;
    private DisplaySurface dsurf;
    private GraphLayout graphLayout;
    private boolean showNetSequenceGraph;
    private NetSequenceGraph netSequenceGraph;
    private int layout;

    public ModelGUI() {

        // Setting the order of appearance of parameters
        // to NON-alphabetic
        Controller.ALPHA_ORDER = false;
        Controller.CONSOLE_ERR = false;
        Controller.CONSOLE_OUT = false;

        showPowerPlot = true;
        showGraph = true;
        showNetSequenceGraph = false;
    }

    public void setup() {

        // Initializing the original model first
        super.setup();

        // Specify the parameters to be displayed for setting by the user
        params = new String[]{"initNodes", "m", "showPowerPlot", "showGraph", "layout",
                              "showNetSequenceGraph", "pRemoving", "removingStarts"};

        layout = LAYOUT_FRUCHT;

        Hashtable layoutHashtable = new Hashtable();
        layoutHashtable.put(new Integer(LAYOUT_CIRCULAR), "Circular");
        layoutHashtable.put(new Integer(LAYOUT_RANDOM), "Random");
        layoutHashtable.put(new Integer(LAYOUT_FRUCHT), "Fruchterman");
        layoutHashtable.put(new Integer(LAYOUT_KAMADA), "Kamada");
        ListPropertyDescriptor pdLayout = new ListPropertyDescriptor("Layout", layoutHashtable);
        descriptors.put("Layout", pdLayout);

        // Initializing the GUI-part
        // If there are already graphics objects from a previous run, we need
        // to delete them to clean up the screen.
        if (powerPlot != null)
            powerPlot.dispose();

        if (netSequenceGraph != null) {
            netSequenceGraph.dispose();
        }

        if (dsurf != null)
            dsurf.dispose();

        // We create a new display surface
        dsurf = new DisplaySurface(this, "Network Display");
        // Registering it as part of our GUI
        registerDisplaySurface("Main", dsurf);

        System.gc();
    }

    public void buildModel() {
        // Build the original model first
        super.buildModel();

        // We create and display the plot area
        if (showPowerPlot) {
            powerPlot = new Plot("Degree Distribution");
            powerPlot.display();
        }

        if(showNetSequenceGraph) {
            netSequenceGraph = new NetSequenceGraph("Chart", this, agentList);
            netSequenceGraph.setYRange(0, 5);
            netSequenceGraph.display();
        }

        // We create the network display
        // This involves creating a graphLayout scheme and passing it to the
        // newly created display itself.
        if (showGraph) {
            switch(layout) {
                case LAYOUT_RANDOM:
                    graphLayout = new RandomGraphLayout(agentList, 500, 500);
                    break;
                case LAYOUT_CIRCULAR:
                    graphLayout = new CircularGraphLayout(agentList, 500, 500);
                    break;
                case LAYOUT_FRUCHT:
                    graphLayout = new FruchGraphLayout(agentList, 500, 500, dsurf, 0);
                    break;
                case LAYOUT_KAMADA:
                    graphLayout = new KamadaGraphLayout(agentList, 500, 500, dsurf, 0);
                    break;
            }
            Network2DDisplay display = new Network2DDisplay(graphLayout);

            // We add and display the new display.
            dsurf.addDisplayableProbeable(display, "Network Display");
            addSimEventListener(dsurf);
            dsurf.display();
        }
    }

    /////////////////////////////////////////////////////////////////////////
    //  Iterated methods ////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * This method calculates the ratio of agents having a
     * degree value equal to 'size'.
     *
     * @param size the degree threshold
     * @return the ratio of agents
     */
    protected double calcProb(int size) {
        int n = 0;
        Iterator i = agentList.iterator();
        while (i.hasNext()) {
            DefaultNode node = (DefaultNode) i.next();
            if (node.getInDegree() + node.getOutDegree() >= size) {
                n++;
            }
        }
        return (double) (n + 1) / (double) agentList.size();
    }

    /**
     * This method updates the 'log-log graph' of degree distibutions.
     */
    protected void plotPoints() {
        int linePoints = 0;
        double x1 = 1000.0;
        double x2 = 0.0;
        double a, b, sx, sy, sxx, sxy;
        sx = 0.0;
        sy = 0.0;
        sxx = 0.0;
        sxy = 0.0;

        powerPlot.clear(0);
        powerPlot.clear(1);
        powerPlot.display();
        powerPlot.setConnected(false, 0);

        Iterator i = agentList.iterator();
        while (i.hasNext()) {
            DefaultNode n = (DefaultNode) i.next();
            if (n.getInDegree() + n.getOutDegree() > 0) {
                double x = Math.log((double) n.getInDegree() + n.getOutDegree()) /
                        Math.log(10.0);
                double y = Math.log(calcProb(n.getInDegree() + n.getOutDegree())) /
                        Math.log(10.0);
                powerPlot.plotPoint(x, y, 0);
                linePoints++;
                sx = sx + x;
                sy = sy + y;
                sxx = sxx + x * x;
                sxy = sxy + x * y;

                if (x < x1)
                    x1 = x;
                if (x > x2)
                    x2 = x;
            }
        }

        b = (linePoints * sxy - sx * sy) / (linePoints * sxx - sx * sx);
        a = (sy - b * sx) / linePoints;
        powerPlot.setConnected(true, 1);
        powerPlot.plotPoint(x1, a + b * x1, 1);
        powerPlot.plotPoint(x2, a + b * x2, 1);

        powerPlot.updateGraph();
        powerPlot.fillPlot();
    }

    public void step() {
        // Do the original model's step() method first
        super.step();

        // Do the GUI-part of it by updating the graph and the display
        if (showPowerPlot)
            plotPoints();

        if(showNetSequenceGraph)
            netSequenceGraph.step();

        if (showGraph) {
            graphLayout.setList(agentList);
            graphLayout.updateLayout();
            dsurf.updateDisplay();
        }
    }

    /**
     * Suppresses postStep() of the Model
     */
    public void postStep() {
    }

    /////////////////////////////////////////////////////////////////////////
    // RePast Parameter Panel Methods ///////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    // In the following we provide get/set methods for all the parameters
    // we listed in params (see the setup() method)

    // Note that we removed numPlayers from here, too, as that's not
    // listed in params anymore.

    public int getInitNodes() {
        return initNodes;
    }

    public void setInitNodes(int n) {
        initNodes = n;
    }

    public int getM() {
        return m;
    }

    public void setM(int m) {
        this.m = m;
    }

    public double getPRemoving() {
        return pRemoving;
    }

    public void setPRemoving(double d) {
        pRemoving = d;
    }

    public int getRemovingStarts() {
        return removingStarts;
    }

    public void setRemovingStarts(int a) {
        removingStarts = a;
    }

    public boolean getShowPowerPlot() {
        return showPowerPlot;
    }

    public void setShowPowerPlot(boolean show) {
        showPowerPlot = show;
    }

    public boolean getShowGraph() {
        return showGraph;
    }

    public void setShowGraph(boolean show) {
        showGraph = show;
    }

    public boolean isShowNetSequenceGraph() {
        return showNetSequenceGraph;
    }

    public void setShowNetSequenceGraph(boolean showNetSequenceGraph) {
        this.showNetSequenceGraph = showNetSequenceGraph;
    }

    public int getLayout() {
        return layout;
    }

    public void setLayout(int layout) {
        this.layout = layout;
    }

   public static void main(String[] args) {
        SimInit init = new SimInit();
        // We MUST create a ModelGUI object instead of an instance of Model
        // as in the parent class, in order to get the GUI functionality.
        Model m = new ModelGUI();
        init.loadModel(m, null, false);
    }
}